@@ -1,6 +1,6 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
-rpl_ioctl.h - IOCTL numbers for `ttyrpld` and the rpldev module
+include/rpl_ioctl.h - IOCTL numbers for RPLD and RPLDEV
   Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
   -- License restrictions apply (GPL2)
 
@@ -33,9 +33,11 @@ rpl_ioctl.h - IOCTL numbers for `ttyrpld` and the rpldev module
 enum {
     RPL_IOC_MAGIC      = 0xB7,
     RPL_IOC_GETBUFSIZE = _IOR(RPL_IOC_MAGIC, 1, size_t),
-    RPL_IOC__MAXNUM    = _IO(RPL_IOC_MAGIC, 2),
+    RPL_IOC_GETRAVAIL  = _IOR(RPL_IOC_MAGIC, 2, size_t),
+    RPL_IOC_GETWAVAIL  = _IOR(RPL_IOC_MAGIC, 3, size_t),
+    RPL_IOC__MAXNUM    = _IO(RPL_IOC_MAGIC, 4),
 };
 
-#endif /* RPL_IOCTL_H */
+#endif // RPL_IOCTL_H
 
 //==[ End of file ]============================================================
