@@ -1,6 +1,6 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
-packet.h - Packet definition for `ttyrpld` and the rpldev module
+include/rpl_packet.h - Packet definitions for RPLD and RPLDEV
   Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
   -- License restrictions apply (GPL2)
 
