@@ -1,6 +1,6 @@
 /*=============================================================================
 ttyrpld - TTY replay daemon
-kernel/rpldev.c - Kernel interface for `ttyrpld`
+kernel/rpldev.c - Kernel interface for RPLD
   Copyright (C) Jan Engelhardt <jengelh [at] linux01 gwdg de>, 2004
   -- License restrictions apply (GPL2)
 
@@ -21,6 +21,7 @@ kernel/rpldev.c - Kernel interface for `ttyrpld`
 
   -- For details see doc/GPL2.txt.
 =============================================================================*/
+#include <asm/byteorder.h>
 #include <asm/semaphore.h>
 #include <asm/uaccess.h>
 #include <linux/fs.h>
@@ -30,11 +31,19 @@ kernel/rpldev.c - Kernel interface for `ttyrpld`
 #include <linux/module.h>
 #include <linux/poll.h>
 #include <linux/rpl.h>
-#include <linux/slab.h>
 #include <linux/string.h>
 #include <linux/wait.h>
-
 #include <linux/version.h>
+#include <linux/vmalloc.h>
+#include "../include/rpl_ioctl.h"
+#include "../include/rpl_packet.h"
+
+#ifdef __LITTLE_ENDIAN
+#    define PP_swab(ptr)
+#else
+#    define PP_swab(ptr) my_swab((ptr), sizeof(*(ptr)))
+#endif
+
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #    define KERNEL_2_4 1
 #    define P_MAGIC MAGIC_2_4
@@ -42,8 +51,6 @@ kernel/rpldev.c - Kernel interface for `ttyrpld`
 #    define P_MAGIC MAGIC_2_6
 #endif
 
-#include "../include/rpl_ioctl.h"
-#include "../include/rpl_packet.h"
 #ifndef KBUILD_BASENAME
 #    define KBUILD_BASENAME "rpldev"
 #endif
@@ -71,18 +78,19 @@ static int uif_close(struct inode *, struct file *);
 // Local functions
 inline static ssize_t avail_R(void);
 inline static ssize_t avail_W(void);
-inline static int imin(int, int);
+inline static unsigned int imin_uint(unsigned int, unsigned int);
 inline static void get_tty_devnr(uint32_t *, struct tty_struct *);
 static int mv_buffer(struct rpld_packet *, const char *, size_t);
 inline static void mv_buffer2(const void *, size_t);
 inline static void mv_user(void *, size_t);
+inline static void my_swab(void *, size_t);
 
 // Variables
 static DECLARE_WAIT_QUEUE_HEAD(GV_inq);
 static DECLARE_MUTEX(GV_open_lock);
 static DECLARE_MUTEX(GV_buf_lock);
 static char *GV_buffer = NULL, *GV_bufr = NULL, *GV_bufw = NULL;
-static size_t GI_bufsize = 16384; // TTY driver has 4096
+static size_t GI_bufsize = 16384; // Each tty has a buffer of 4096
 static int GI_minor = MISC_DYNAMIC_MINOR;
 static struct file_operations GI_fops = {
     open:    uif_open,
@@ -134,6 +142,8 @@ static int krn_open(struct tty_struct *tty, struct tty_struct *cur_tty) {
     p.event = EV_OPEN;
     p.magic = P_MAGIC;
     p.size  = 0;
+    PP_swab(&p.dev);
+    PP_swab(&p.size);
     return mv_buffer(&p, NULL, 0);
 }
 
@@ -146,6 +156,8 @@ static int krn_read(const char *buf, size_t count, struct tty_struct *tty) {
     p.event = EV_READ;
     p.magic = P_MAGIC;
     p.size  = count;
+    PP_swab(&p.dev);
+    PP_swab(&p.size);
     return mv_buffer(&p, buf, count);
 }
 
@@ -156,6 +168,8 @@ static int krn_write(const char *buf, size_t count, struct tty_struct *tty) {
     p.event = EV_WRITE;
     p.magic = P_MAGIC;
     p.size  = count;
+    PP_swab(&p.dev);
+    PP_swab(&p.size);
     return mv_buffer(&p, buf, count);
 }
 
@@ -169,6 +183,8 @@ static int krn_ioctl(struct tty_struct *tty, struct tty_struct *real_tty,
     p.magic = P_MAGIC;
     p.size  = sizeof(buf);
 
+    PP_swab(&p.dev);
+    PP_swab(&p.size);
     memcpy(buf, &cmd, sizeof(cmd));
     memcpy(buf + sizeof(cmd), &arg, sizeof(arg));
     return mv_buffer(&p, buf, sizeof(buf));
@@ -180,6 +196,8 @@ static int krn_close(struct tty_struct *tty) {
     p.event = EV_CLOSE;
     p.magic = P_MAGIC;
     p.size  = 0;
+    PP_swab(&p.dev);
+    PP_swab(&p.size);
     return mv_buffer(&p, NULL, 0);
 }
 
@@ -199,7 +217,11 @@ static int uif_open(struct inode *inode, struct file *filp) {
     packets could go to different readers. */
     if(down_trylock(&GV_open_lock) != 0) { return -EBUSY; }
 
-    if((GV_buffer = kmalloc(GI_bufsize, GFP_USER)) == NULL) { return -ENOMEM; }
+    GV_buffer = __vmalloc(GI_bufsize, GFP_USER | __GFP_HIGHMEM, PAGE_KERNEL);
+    if(GV_buffer == NULL) {
+        up(&GV_open_lock);
+        return -ENOMEM;
+    }
     GV_bufr = GV_bufw = GV_buffer;
 
     /* Update links. I do it here 'cause I do not want memory copying (from the
@@ -223,13 +245,13 @@ static int uif_open(struct inode *inode, struct file *filp) {
 
 static ssize_t uif_read(struct file *filp, char *buf, size_t count,
  loff_t *ppos) {
+    // Nothing read, nothing done
+    if(count == 0) { return 0; }
+
     /* No seeking allowed. No positional reading allowed either. For skipping
     data, use SEEK_CUR with lseek(). */
     if(ppos != &filp->f_pos) { return -ESPIPE; }
 
-    // Nothing read, nothing done
-    if(count == 0) { return 0; }
-
     // Must sleep as long as there is no data.
     if(down_interruptible(&GV_buf_lock)) { return -ERESTARTSYS; }
     while(GV_bufr == GV_bufw) {
@@ -242,7 +264,7 @@ static ssize_t uif_read(struct file *filp, char *buf, size_t count,
     }
 
     // Data is available, so give it to the user
-    count = imin(count, avail_R());
+    count = imin_uint(count, avail_R());
     mv_user(buf, count);
 
     up(&GV_buf_lock);
@@ -259,7 +281,7 @@ static loff_t uif_llseek(struct file *filp, loff_t offset, int origin) {
     if(offset == 0) { return 0; }
 
     if(down_interruptible(&GV_buf_lock)) { return -ERESTARTSYS; }
-    GV_bufr += imin(offset, avail_R());
+    GV_bufr += imin_uint(offset, avail_R());
     if(GV_bufr > GV_buffer + GI_bufsize) { GV_bufr -= GI_bufsize; }
 
     up(&GV_buf_lock);
@@ -273,6 +295,10 @@ static int uif_ioctl(struct inode *inode, struct file *filp, unsigned int cmd,
     switch(cmd) {
         case RPL_IOC_GETBUFSIZE:
             return GI_bufsize;
+        case RPL_IOC_GETRAVAIL:
+            return avail_R();
+        case RPL_IOC_GETWAVAIL:
+            return avail_W();
         default:
             return -ENOTTY;
     }
@@ -291,7 +317,7 @@ static int uif_close(struct inode *inode, struct file *filp) {
     rpl_qwrite = NULL;
     rpl_qioctl = NULL;
     rpl_qclose = NULL;
-    kfree(GV_buffer);
+    vfree(GV_buffer);
     up(&GV_open_lock);
     return 0;
 }
@@ -318,7 +344,7 @@ inline static void get_tty_devnr(uint32_t *dev, struct tty_struct *tty) {
     return;
 }
 
-inline static int imin(int a, int b) {
+inline static unsigned int imin_uint(unsigned int a, unsigned int b) {
     return a < b ? a : b;
 }
 
@@ -368,12 +394,12 @@ inline static void mv_buffer2(const void *src, size_t count) {
 }
 
 inline static void mv_user(void *dest, size_t count) {
-    /* mv_user() reads <count> bytes from the circulary buffer and puts it into
+    /* mv_user() reads COUNT bytes from the circulary buffer and puts it into
     userspace memory. */
     int x = GV_buffer + GI_bufsize - GV_bufr;
 
     /* Since this operation may block due to userspace page faults, the
-    userspace daemon should really have <dest> memory-locked. */
+    userspace daemon should really have DEST memory-locked. */
     if(count <= x) {
         copy_to_user(dest, GV_bufr, count);
         GV_bufr += count;
@@ -386,4 +412,19 @@ inline static void mv_user(void *dest, size_t count) {
     return;
 }
 
+void my_swab(void *srcp, size_t count) {
+    if(count == 16) {
+        uint16_t *ptr = srcp;
+        *ptr = cpu_to_le16(*ptr);
+    } else if(count == 32) {
+        uint32_t *ptr = srcp;
+        *ptr = cpu_to_le32(*ptr);
+    } else if(count == 64) {
+        uint64_t *ptr = srcp;
+        *ptr = cpu_to_le64(*ptr);
+    }
+
+    return;
+}
+
 //==[ End of file ]============================================================
