@@ -36,7 +36,7 @@ user/replay.c - Realtime TTY log analyzer
 
 #include "dev.h"
 #include "rpl_packet.h"
-#include "ushared.h"
+#include "global.h"
 
 #define offsetof(type, member) ((size_t)&((type *)NULL)->member)
 #define containerof(var, type, member) ((void *)(var) - offsetof(type, member))
@@ -59,19 +59,16 @@ enum { // skip types
 
 struct {
     double factor;
-    int follow, nodelay;
-    long ovcorr;
-    struct {
-        long val;
-        int type;
-    } seek_to;
+    int follow, nodelay, seek_type;
+    long ovcorr, maxdelay;
+    long seek_cval;
 } Opt = {
-    .factor   = 1.0,
-    .follow   = 0,
-    .nodelay  = 0,
-    .seek_to  = {
-        .type = SKIP_NONE,
-    },
+    .factor    = 1.0,
+    .follow    = 0,
+    .maxdelay  = -1,
+    .nodelay   = 0,
+    .seek_type = SKIP_NONE,
+    .seek_cval = 0,
 };
 
 static void replay_file(int);
@@ -94,7 +91,7 @@ int main(int argc, char **argv) {
     if(!get_options(&argc, (const char ***)&argv)) { return EXIT_FAILURE; }
 
     fprintf(stderr,
-      "> ttyreplay\n"
+      "> ttyreplay " TTYRPLD_VERSION "\n"
       "This program comes with ABSOLUTELY NO WARRANTY; it is free software\n"
       "and you are welcome to redistribute it under certain conditions;\n"
       "for details see the doc/GPL2.txt file which should have come with\n"
@@ -128,11 +125,7 @@ static void replay_file(int fd) {
 
     if(Opt.follow == FM_LIVE && !seek_to_end(fd)) {
         return;
-    }/* else {
-        if(Opt.seek_to.type == SKIP_WRPACKET) {
-            
-        }
-    }*/
+    }
 
     while((eax = read_waitfm(fd, &packet, sizeof(struct disk_packet))) ==
      sizeof(struct disk_packet)) {
@@ -148,21 +141,49 @@ static void replay_file(int fd) {
             continue;
         }
 
+        PP_swab(&packet.size, sizeof(packet.size));
+        PP_swab(&packet.tv.tv_sec, sizeof(packet.tv.tv_sec));
+        PP_swab(&packet.tv.tv_usec, sizeof(packet.tv.tv_usec));
+
         switch(packet.event) {
             case EV_WRITE:
+                if(Opt.seek_type == SKIP_WRPACKET && Opt.seek_cval-- > 0) {
+                    read_nullfm(fd, packet.size);
+                    break;
+                }
                 if(!tick) {
                     // No delay after the first packet has been read...
                     ++tick;
+                    if(Opt.seek_cval > 0) {
+                        memcpy(&stamp, &packet.tv, sizeof(struct timeval));
+                        read_nullfm(fd, packet.size);
+                        break;
+                    }
                 } else if(Opt.follow != FM_LIVE && Opt.follow != FM_NODELAY) {
                     /* ... only when we know the next packet and can calculate
                     the time difference. We do not execute this code in FM_LIVE
                     mode, because we are already waiting enough with
-                    SH_read_wait(). In fact, allowing this code block for
-                    FM_LIVE creates a skew! */
+                    read_wait(). In fact, allowing this code block for FM_LIVE
+                    creates a skew because of the usleep_ovcorr()! */
+                    unsigned long long x;
                     tv_delta(&stamp, &packet.tv, &delta);
+                    if(Opt.seek_type == SKIP_TIME) {
+                        if((Opt.seek_cval -= tv2usec(&delta) / 1000) > 0) {
+                            memcpy(&stamp, &packet.tv, sizeof(struct timeval));
+                            read_nullfm(fd, packet.size);
+                            break;
+                        }
+                    }
                     if(Opt.factor != 1.0) {
+                        // -S switch for the fast
                         usec2tv(tv2usec(&delta) / Opt.factor, &delta);
                     }
+                    if(Opt.maxdelay > 0 &&
+                     (x = tv2usec(&delta)) > Opt.maxdelay) {
+                        /* -m switch for those who do not want to wait that
+                        long. */
+                        usec2tv(Opt.maxdelay, &delta);
+                    }
                     usleep_ovcorr(&delta, &skew);
                 }
                 memcpy(&stamp, &packet.tv, sizeof(struct timeval));
@@ -297,8 +318,10 @@ static int find_next_packet(int fd) {
         /* Mmhkay, there is another header other than the current (packet)
         in the buffer. Crap, another one gone. */
         memmove(buf, buf + LZ + s, BZ - LZ - s);
-        if(read_waitfm(fd, buf + BZ - LZ - s, s) < s) { return 0; }
-        if(!read_nullfm(fd, packet->size)) { return 0; }
+        if(read_waitfm(fd, buf + BZ - LZ - s, s) < s ||
+         !read_nullfm(fd, packet->size)) {
+            return 0;
+        }
     } else {
         // Just subtract what we have already read into the buffer
         if(!read_nullfm(fd, s - LZ)) { return 0; }
@@ -312,21 +335,23 @@ static int find_next_packet(int fd) {
 
 static int get_options(int *argc, const char ***argv) {
     static const char *_empty_argv[] = {NULL, NULL};
-    char *tmp;
+    char *tmp = NULL;
     struct poptOption options_table[] = {
         {NULL, 'F', POPT_ARG_NONE, NULL, 'F',
          "Live feed follow mode (like `tail -f`)", NULL},
-/*        {NULL, 'J', POPT_ARG_STRING, &tmp, 'J',
+        {NULL, 'J', POPT_ARG_STRING, &tmp, 'J',
          "Seek to time relative to the start of the log"
-         " (HH:MM:SS or total SEC)", "tspec"},*/
+         " (HH:MM:SS or total SEC, see docs)", "tspec"},
         {NULL, 'S', POPT_ARG_DOUBLE, &Opt.factor, '\0',
          "Speed factor (default: 1.0)", NULL},
         {NULL, 'T', POPT_ARG_NONE, &Opt.nodelay, 'T',
          "No delays (useful for text file grepping)", NULL},
         {NULL, 'f', POPT_ARG_NONE, NULL, 'f',
          "Catch-up follow mode (play file, switch to live feed on EOF)", NULL},
-/*        {NULL, 'j', POPT_ARG_LONG, &Opt.seek_to.val, 'j',
-         "Skip the given number of EV_WRITE packets", NULL},*/
+        {NULL, 'j', POPT_ARG_STRING, &tmp, 'j',
+         "Skip the given number of EV_WRITE packets", NULL},
+        {NULL, 'm', POPT_ARG_LONG, &Opt.maxdelay, 'm',
+         "Maximum delay to wait between packets", "msec"},
         POPT_AUTOHELP
         POPT_TABLEEND
     };
@@ -344,8 +369,8 @@ static int get_options(int *argc, const char ***argv) {
             case 'J': {
                 char *sec = "", *min = "", *hr = "";
 
-                Opt.seek_to.type = SKIP_TIME;
-                Opt.seek_to.val  = 0;
+                Opt.seek_type = SKIP_TIME;
+                Opt.seek_cval = 0;
 
                 if((sec = strrchr(tmp, ':')) == NULL) {
                     sec = tmp;
@@ -358,8 +383,9 @@ static int get_options(int *argc, const char ***argv) {
                         hr = tmp;
                     }
                 }
-                Opt.seek_to.val = strtoul(sec, NULL, 0) +
-                 60 * strtoul(min, NULL, 0) + 3600 * strtoul(hr, NULL, 0);
+                Opt.seek_cval = strtol(sec, NULL, 0) +
+                 60 * strtol(min, NULL, 0) + 3600 * strtol(hr, NULL, 0);
+                Opt.seek_cval *= 1000;
                 free(tmp);
                 break;
             }
@@ -370,7 +396,12 @@ static int get_options(int *argc, const char ***argv) {
                 Opt.follow = FM_CATCHUP;
                 break;
             case 'j':
-                Opt.seek_to.type = SKIP_WRPACKET;
+                Opt.seek_type = SKIP_WRPACKET;
+                Opt.seek_cval = strtoll(tmp, NULL, 0);
+                free(tmp);
+                break;
+            case 'm':
+                Opt.maxdelay *= 1000;
                 break;
         }
     }
@@ -382,7 +413,7 @@ static int get_options(int *argc, const char ***argv) {
     }
 
     if((args = poptGetArgs(ctx)) != NULL) {
-        argk = SH_count_args(args);
+        argk = count_args(args);
         poptDupArgv(argk, args, argc, argv);
     } else {
         *argc = 0;
@@ -394,7 +425,9 @@ static int get_options(int *argc, const char ***argv) {
 }
 
 static ssize_t read_through(int in, int out, size_t count) {
-    // Read from IN and directly give it to OUT. Like sendfile().
+    /* Read from IN and directly give it to OUT. This is like sendfile(),
+    but does not use it, since it gives EINVAL most of the times, probably
+    because "out" is a tty and does not seem to support sendpage. */
     char buf[BUFSIZE];
     size_t rem = count;
 
@@ -411,11 +444,12 @@ static ssize_t read_through(int in, int out, size_t count) {
 }
 
 inline static ssize_t read_nullfm(int fd, size_t count) {
-    return SH_skip(fd, count, Opt.follow != FM_NONE && Opt.follow != FM_NODELAY);
+    return G_skip(fd, count, Opt.follow != FM_NONE &&
+     Opt.follow != FM_NODELAY);
 }
 
 static ssize_t read_waitfm(int fd, void *buf, size_t count) {
-    /* Wrapper function for either read() or SH_read_wait(). If EOF has been
+    /* Wrapper function for either read() or read_wait(). If EOF has been
     detected (ATM), we switch into FM_LIVE mode (if currently in FM_CATCHUP
     mode). */
     struct stat sb;
@@ -437,7 +471,7 @@ static ssize_t read_waitfm(int fd, void *buf, size_t count) {
         Opt.follow = FM_LIVE;
     }
 
-    return SH_read_wait(fd, buf, count);
+    return read_wait(fd, buf, count);
 }
 
 static int seek_to_end(int fd) {
