@@ -26,13 +26,12 @@ user/rpld.c - User space daemon (filtering, etc.)
 #include <sys/time.h> // gettimeofday()
 #include <sys/types.h>
 #include <ctype.h>
-#include <endian.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <popt.h>
+#include <pthread.h>
 #include <pwd.h>
 #include <signal.h>
-#include <stdarg.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -46,29 +45,19 @@ extern int posix_memalign(void **, size_t, size_t);
 #include "dev.h"
 #include "rpl_ioctl.h"
 #include "rpl_packet.h"
-#include "ushared.h"
-
-enum {
-    NF_NEWLINE = 1 << 0,
-};
-
-struct tty {
-    uint32_t dev;
-    int fd;
-};
+#include "global.h"
+#include "rdsh.h"
+#define NODEPATH "/var/run/.rpldev"
 
 // The prototype order is the same as the function body order
 static void mainloop(int);
 
 static int packet_preaction(struct mem_packet *);
 static int packet_action(struct mem_packet *, struct tty *, int);
+static int packet_postaction(struct mem_packet *, struct tty *);
 
-static char *devname(uint32_t, char *, size_t);
-static char *devname_fs(uint32_t, char *, size_t);
-static struct tty *get_tty(uint32_t);
 static int log_open(struct tty *);
 static void log_write(struct mem_packet *, struct tty *, int);
-static void log_close(struct tty *);
 
 static int init_device(const char *);
 static int init_sbuffer(int);
@@ -77,67 +66,52 @@ static const char *mknod_rpldev(void);
 static int init_sighandler(void);
 static void sighandler_int(int);
 static void sighandler_alrm(int);
+static void sighandler_pipe(int);
 
 static int get_options(int *, const char ***);
 static int read_config(const char *);
 static int read_config_bp(const char *, const char *);
 static int pconfig_user(const char *, unsigned char, void *, void *);
 
+inline static const char *basename(const char *);
 static char *getnamefromuid(uid_t, char *, size_t);
 static uid_t getuidfromname(const char *);
-static int notify(int, unsigned int, const char *, ...);
 static char *replace(char *, const char *, size_t, char **);
-static int value_cmp(void *, void *);
+inline static int value_cmp(void *, void *);
 
 // Global variables (begin with uppercase character)
-static struct {
-    char *_version, *ovdev, *fbdev, *ofmt;
-    int nocreat, syslog, verbose;
-    long bsize, user_id;
-} Opt = {
-    ._version = "0.85",
-    .bsize    = 16384,
-    .ovdev    = NULL,
-    .fbdev    = "/dev/rpl:/dev/misc/rpl",
-    .nocreat  = 0,
-    .ofmt     = "%u@%d.%t.%l",
-    .syslog   = 0,
-    .verbose  = 0,
-    .user_id  = -1,
-};
-
-struct {
-    size_t open, read, write, ioctl, close, badpack;
-    unsigned long long in, out;
-} Stats;
-
-static struct HXbtree *Ttys = NULL;
 static void *Buffer = NULL;
-static int Running = 1;
 
 //-----------------------------------------------------------------------------
 int main(int argc, const char **argv) {
+    pthread_t infod_id;
     int fd;
 
-    if(!read_config_bp(*argv, "rpld.conf") || !get_options(&argc, &argv)) {
+    /* Yep, the config file is what is needed by all three
+    (rpld, infod, rplctl). */
+    if(!read_config_bp(*argv, "rpld.conf")) {
         return EXIT_FAILURE;
     }
 
+    if(strcmp(basename(*argv), "rplctl") == 0) {
+        return rplctl_main(argc, argv);
+    }
+
+    if(!get_options(&argc, &argv)) { return EXIT_FAILURE; }
+    memset(&Stats, 0, sizeof(Stats));
+
     if(Opt.verbose) {
-        memset(&Stats, 0, sizeof(Stats));
         printf(
-          "# rpld %s\n"
+          "# rpld " TTYRPLD_VERSION "\n"
           "This program comes with ABSOLUTELY NO WARRANTY; it is free"
           " software\n"
           "and you are welcome to redistribute it under certain conditions;\n"
           "for details see the doc/GPL2.txt file which should have come with\n"
-          "this program.\n\n",
-          Opt._version
+          "this program.\n\n"
         );
     }
 
-    SH_check_balign();
-    if((Ttys = HXbtree_init(HXBT_ASSOC | HXBT_LKEY | HXBT_FCMP,
+    if((Ttys = HXbtree_init(HXBT_ASSOC | HXBT_LKEY | HXBT_FCMP | HXBT_XCPY,
      value_cmp)) == NULL) {
         perror("HXbtree_init()");
         return EXIT_FAILURE;
@@ -154,12 +128,23 @@ int main(int argc, const char **argv) {
     // Callback tick for statistics
     if(Opt.verbose) { alarm(1); }
 
-    // Not seteuid(), because it shall not be posibble to regain privs.
-    if(Opt.user_id >= 0) { setuid(Opt.user_id); }
+    if(Opt.infod_start) { infod_init(); }
 
     if(Opt.syslog) { openlog("rpld", LOG_PID, LOG_DAEMON); }
 
+    // Not seteuid(), because it shall not be posibble to regain privs.
+    if(Opt.user_id >= 0) { setuid(Opt.user_id); }
+
+    if(Opt.infod_start) { pthread_create(&infod_id, NULL, infod_main, NULL); }
+
     mainloop(fd);
+
+    if(Opt.infod_start) {
+        unlink(Opt.infod_port);
+        pthread_cancel(infod_id);
+        pthread_join(infod_id, NULL);
+    }
+
     close(fd);
     deinit_sbuffer();
     return EXIT_SUCCESS;
@@ -167,27 +152,28 @@ int main(int argc, const char **argv) {
 
 static void mainloop(int fd) {
     ssize_t (*reader)(int, void *, size_t) = read;
-    int pk = 0;
 
     { /* read() on the device might return less than we request, so we use
-      SH_read_wait() and loop until there is data. However, we do not want to
-      wait on e.g. pipes. */
+      read_wait() and loop until there is data. However, we do not want to wait
+      on e.g. pipes. */
       struct stat sb;
       fstat(fd, &sb);
-      if(S_ISCHR(sb.st_mode)) { reader = SH_read_wait; }
+      if(S_ISCHR(sb.st_mode)) { reader = read_wait; }
     }
 
-    while(Running) {
+    while(Opt._running) {
         struct mem_packet packet;
         struct tty *tty;
 
-        errno = 0;
         if(reader(fd, &packet, sizeof(struct rpld_packet)) <
          sizeof(struct rpld_packet)) {
-            Running = 0;
+            Opt._running = 0;
             break;
         }
-++pk;
+
+        PP_swab(&packet.dev, sizeof(packet.dev));
+        PP_swab(&packet.size, sizeof(packet.size));
+
         if(packet.magic == MAGIC_2_6) {
             // nothing needed
         } else if(packet.magic == MAGIC_2_4) {
@@ -196,8 +182,8 @@ static void mainloop(int fd) {
              K24_MINOR(packet.dev));
         } else {
             ++Stats.badpack;
-            notify(LOG_WARNING, NF_NEWLINE, "Bogus packet (magic is 0x%X) pack%d!\n",
-             packet.magic, pk);
+            notify(LOG_WARNING, NF_NEWLINE,
+             "Bogus packet (magic is 0x%X)!\n", packet.magic);
             continue;
         }
 
@@ -205,18 +191,23 @@ static void mainloop(int fd) {
         the amount of data which has to be copied over the device. */
         gettimeofday(&packet.tv, NULL);
 
-        if(!packet_preaction(&packet) || (tty = get_tty(packet.dev)) == NULL) {
-            SH_skip(fd, packet.size, 0);
+        if(!packet_preaction(&packet)) {
+            G_skip(fd, packet.size, 0);
+            continue;
+        }
+
+        pthread_mutex_lock(&Ttys_lock);
+        tty = get_tty(packet.dev);
+        pthread_mutex_unlock(&Ttys_lock);
+
+        if(tty == NULL || !packet_postaction(&packet, tty)) {
+            G_skip(fd, packet.size, 0);
             continue;
         }
 
         packet_action(&packet, tty, fd);
     }
 
-    if(Opt.verbose) {
-        if(errno == 0) { fprintf(stderr, "<EOF on input>\n"); }
-        else { perror("read()"); }
-    }
     return;
 }
 
@@ -226,18 +217,16 @@ static int packet_preaction(struct mem_packet *packet) {
     so no need to record them too. */
     register long maj = K26_MAJOR(packet->dev);
 
-    if(maj == 2 /* BSD ptm (/dev/ptypXY) */) {
-        if(packet->event != EV_CLOSE) { return 0; }
-        packet->dev = K26_MKDEV(K26_MAJOR(3), K26_MINOR(packet->dev));
-    }
-
     if(maj >= 128 && maj < 136 /* Unix98 ptm (-) */) {
         if(packet->event != EV_CLOSE) { return 0; }
-        /* When closing an xterm window or ssh session, the EV_CLOSE event
+        /* When closing an xterm window or SSH session, the EV_CLOSE event
         actually happens on the master side. To keep it easy, it's device
         signature is simply changed to the slave. */
         packet->dev = K26_MKDEV(K26_MAJOR(packet->dev) + 8,
          K26_MINOR(packet->dev));
+    } else if(maj == 2 /* BSD ptm (/dev/ptypXY) */) {
+        if(packet->event != EV_CLOSE) { return 0; }
+        packet->dev = K26_MKDEV(K26_MAJOR(3), K26_MINOR(packet->dev));
     }
 
     /* The excluded ptm packets above are not counted at all, whereas
@@ -269,7 +258,8 @@ static int packet_preaction(struct mem_packet *packet) {
 
     if(packet->event == EV_IOCTL) {
         /* ioctl requests are actually logged by the rpldev Kernel module,
-        but I do not see a use for them ATM. */
+        and I do see a use for them. But: it's simply not
+        implemented yet ;-) */
         return 0;
     }
 
@@ -291,100 +281,24 @@ static int packet_action(struct mem_packet *packet, struct tty *tty, int fd) {
     return 1;
 }
 
+static int packet_postaction(struct mem_packet *packet, struct tty *tty) {
+    /* The name is wrong when you look at the caller, since postaction()
+    actually comes before packet_action(). Anyway, it's post the get_tty(). */
+    if(tty->status != IFP_ACTIVATE) {
+        return 0;
+    }
+    switch(packet->event) {
+        case EV_READ:
+            tty->in += packet->size;
+            break;
+        case EV_WRITE:
+            tty->out += packet->size;
+            break;
+    }
+    return 1;
+}
+
 //-----------------------------------------------------------------------------
-static const char *m_idx = "pqrstuvwxyzabcdef", *s_idx = "0123456789abcdef";
-
-static char *devname(uint32_t dev, char *buf, size_t count) {
-    /* This function returns a "simple name" which can be integrated into
-    rpld's log file name scheme. */
-    unsigned long minor = K26_MINOR(dev);
-
-    switch(K26_MAJOR(dev)) {
-        case 2: // BSD pty masters
-            snprintf(buf, count, "pty%c%c",
-             m_idx[minor >> 4], s_idx[minor % 16]);
-            break;
-        case 3: // BSD pty slaves
-            snprintf(buf, count, "tty%c%c",
-             m_idx[minor >> 4], s_idx[minor % 16]);
-            break;
-        case 4:
-            // Virtual console (0-63) and serial console (64-255)
-            if(minor < 64) { snprintf(buf, count, "vc-%lu", minor); }
-            else { snprintf(buf, count, "ttyS%lu", minor - 64); }
-            break;
-        case 5:
-            if(minor == 1) { snprintf(buf, count, "console"); }
-            else { goto unknown; }
-        case 128:
-            // Unix98 pty masters (got no real /dev entry, though, see below)
-            snprintf(buf, count, "ptm-%lu", minor);
-            break;
-        case 136: // Unix98 pty slaves
-            snprintf(buf, count, "pts-%lu", minor);
-            break;
-    }
-    return buf;
-
-unknown:
-    /* Not every char device is a tty, and I even do not want to include a
-    name database for all possible tty devices. */
-    snprintf(buf, count, "%lu-%lu", K26_MAJOR(dev), minor);
-    return buf;
-}
-
-static char *devname_fs(uint32_t dev, char *buf, size_t count) {
-    /* This function returns a device filename generated from major/minor
-    number, which is then used for stat(). */
-    unsigned long minor = K26_MINOR(dev);
-
-    switch(K26_MAJOR(dev)) {
-        case 2: // BSD pty masters
-            snprintf(buf, count, "/dev/pty%c%c",
-             m_idx[minor >> 4], s_idx[minor % 16]);
-            break;
-        case 3: // BSD pty slaves
-            snprintf(buf, count, "/dev/tty%c%c",
-             m_idx[minor >> 4], s_idx[minor % 16]);
-            break;
-        case 4:
-            // Virtual console (0-63) and serial console (64-255)
-            if(minor < 64) { snprintf(buf, count, "/dev/tty%lu", minor); }
-            else { snprintf(buf, count, "/dev/ttyS%lu", minor - 64); }
-            break;
-        case 5:
-            if(minor == 1) { snprintf(buf, count, "/dev/console"); }
-            else { goto unknown; }
-        case 128:
-            /* Unix98 pty masters. They usually have no device nodes in /dev,
-            and should instead be accessed over the multiplexer /dev/ptmx.
-            However IIRC, devfs popped them up in /dev/ptm. */
-            snprintf(buf, count, "/dev/ptm/%lu", minor);
-            break;
-        case 136: // Unix98 pty slaves
-            snprintf(buf, count, "/dev/pts/%lu", minor);
-            break;
-    }
-    return buf;
-
-unknown:
-    return NULL;
-}
-
-static struct tty *get_tty(uint32_t dev) {
-    struct HXbtree_node *ts;
-    struct tty *ret, *tty;
-    if((ret = HXbtree_get(Ttys, (void *)dev)) != NULL) { return ret; }
-    if((tty = malloc(sizeof(struct tty))) == NULL) { return NULL; }
-    tty->dev = dev;
-    tty->fd  = -1;
-    if((ts = HXbtree_add(Ttys, (void *)dev, tty)) == NULL) {
-        notify(LOG_ERR, NF_NEWLINE, "Memory allocation failure");
-        return NULL;
-    }
-    return ts->data;
-}
-
 static int log_open(struct tty *tty) {
     char buf[MAXFNLEN], devnode[64], fmday[16], fmtime[16], user[32];
     char *subst_catalog[] = {
@@ -397,7 +311,7 @@ static int log_open(struct tty *tty) {
 
     { // User
       struct stat sb;
-      devname_fs(tty->dev, devnode, 64);
+      G_devname_fs(tty->dev, devnode, 64);
       if(stat(devnode, &sb) != 0) {
           strcpy(user, "NONE");
       } else if(getnamefromuid(sb.st_uid, user, 32) == NULL) {
@@ -414,9 +328,11 @@ static int log_open(struct tty *tty) {
     }
 
     // Simple tty name
-    devname(tty->dev, devnode, 64);
+    G_devname_nm(tty->dev, devnode, 64);
     replace(buf, Opt.ofmt, MAXFNLEN, subst_catalog);
 
+    // Open log
+    tty->file = HX_strdup(basename(buf));
     if(Opt.nocreat) { snprintf(buf, MAXFNLEN, "/dev/null"); }
     if((tty->fd = open(buf, O_WRONLY | O_CREAT | O_APPEND, 0600)) < 0) {
         notify(LOG_ERR, NF_NEWLINE, "Could not open %s: %s\n",
@@ -430,11 +346,12 @@ static int log_open(struct tty *tty) {
             .magic = MAGIC_SIG,
             .tv    = {-1, -1},
         };
-        size_t s = snprintf(buf, MAXFNLEN, "ttyrpld %s", Opt._version);
-        p.size = s;
-#ifdef BIGENDIAN
-        SH_swab(&p.size, sizeof(uint16_t));
-#endif
+        size_t s;
+
+        buf[MAXFNLEN - 1] = '\0';
+        strncpy(buf, "ttyrpld " TTYRPLD_VERSION, MAXFNLEN - 1);
+        s = p.size = strlen(buf);
+        PP_swab(&p.size, sizeof(p.size));
         write(tty->fd, &p, sizeof(struct disk_packet));
         write(tty->fd, buf, s);
     }
@@ -443,19 +360,19 @@ static int log_open(struct tty *tty) {
 }
 
 static void log_write(struct mem_packet *packet, struct tty *tty, int fd) {
+    /* Slap a timestamp around the packet and write it out to disk. Open the
+    log if necessary. */
     size_t s = packet->size;
     int read_left;
 
     if(tty->fd == -1) { log_open(tty); }
-#ifdef BIGENDIAN
-    SH_swab(&packet->size, sizeof(packet->size));
-    SH_swab(&packet->tv.tv_sec, sizeof(packet->tv.tv_sec));
-    SH_swab(&packet->tv.tv_usec, sizeof(packet->tv.tv_usec));
-#endif
+    PP_swab(&packet->size, sizeof(packet->size));
+    PP_swab(&packet->tv.tv_sec, sizeof(packet->tv.tv_sec));
+    PP_swab(&packet->tv.tv_usec, sizeof(packet->tv.tv_usec));
     write(tty->fd, &packet->size, sizeof(struct disk_packet));
     read_left = s;
 
-    while(read_left > 0) {
+    while(read_left > 0) { // XXX replace with read_through
         int have_read = read(fd, Buffer, min_uint(read_left, Opt.bsize));
         write(tty->fd, Buffer, have_read);
         read_left -= have_read;
@@ -464,13 +381,6 @@ static void log_write(struct mem_packet *packet, struct tty *tty, int fd) {
     return;
 }
 
-static void log_close(struct tty *tty) {
-    close(tty->fd);
-    tty->fd = -1;
-    HXbtree_del(Ttys, tty);
-    return;
-}
-
 //-----------------------------------------------------------------------------
 static int init_device(const char *in_devs) {
     const char *new_dev = NULL;
@@ -511,7 +421,7 @@ static int init_device(const char *in_devs) {
 static int init_sbuffer(int fd) {
     int eax;
 
-    if((eax = ioctl(fd, RPL_IOC_GETBUFSIZE, 0)) >= 0) {
+    if((eax = ioctl(fd, RPL_IOC_GETBUFSIZE, 0)) > 0) {
         Opt.bsize = eax;
     }
 
@@ -546,7 +456,7 @@ static const char *mknod_rpldev(void) {
     while(fgets(buf, 80, fp) != NULL) {
         char *ptr = buf, *name;
         while(!isdigit(*ptr)) { ++ptr; }
-        minor = strtoul(ptr, NULL, 0);
+        minor = strtol(ptr, NULL, 0);
         while(isdigit(*ptr)) { ++ptr; }
         while(isspace(*ptr)) { ++ptr; }
 
@@ -557,21 +467,24 @@ static const char *mknod_rpldev(void) {
         minor = -1;
     }
 
+    fclose(fp);
+
     if(minor == -1) {
         printf("RPL module does not seem to be loaded\n");
         return NULL;
     }
 
-    if(mknod(".rpl", S_IFCHR | 0400, GLIBC_MKDEV(10, minor)) != 0) {
-        perror("mknod()");
+    if(mknod(NODEPATH, S_IFCHR | 0400, GLIBC_MKDEV(10, minor)) != 0) {
         return NULL;
     }
 
-    return ".rpl";
+    chown(NODEPATH, Opt.user_id, 0);
+    return NODEPATH;
 }
 
 static int init_sighandler(void) {
-    struct sigaction s_int, s_alrm;
+    struct sigaction s_int, s_alrm, s_pipe;
+
     s_int.sa_handler = sighandler_int;
     s_int.sa_flags   = SA_RESTART;
     sigemptyset(&s_int.sa_mask);
@@ -580,16 +493,21 @@ static int init_sighandler(void) {
     s_alrm.sa_flags   = SA_RESTART;
     sigemptyset(&s_alrm.sa_mask);
 
+    s_pipe.sa_handler = sighandler_pipe;
+    s_pipe.sa_flags   = SA_RESTART;
+    sigemptyset(&s_pipe.sa_mask);
+
     /* All sigactions() shall be executed, however, if one fails, this function
     shall return <= 0, otherwise >0 upon success.
     Geesh, I love these constructs. */
     return !(!!sigaction(SIGINT, &s_int, NULL) + 
      !!sigaction(SIGTERM, &s_int, NULL) +
-     !!sigaction(SIGALRM, &s_alrm, NULL));
+     !!sigaction(SIGALRM, &s_alrm, NULL) +
+     !!sigaction(SIGPIPE, &s_pipe, NULL));
 }
 
 static void sighandler_int(int s) {
-    if(Running-- == 0) {
+    if(Opt._running-- == 0) {
         if(Opt.verbose) {
             printf("Second time we received SIGINT/SIGTERM,"
              " exiting immediately.\n");
@@ -597,14 +515,17 @@ static void sighandler_int(int s) {
         exit(EXIT_FAILURE);
     }
     if(Opt.verbose) {
-        printf("\nReceived SIGINT/SIGTERM, shutting down.\n");
+        printf("\n" "Received SIGINT/SIGTERM, shutting down.\n");
+    } else {
+        // If this is not done, process hangs until some fs activity
+        printf("\n");
     }
-    Running = 0;
+    Opt._running = 0;
     return;
 }
 
 static void sighandler_alrm(int s) {
-    fprintf(stderr, "\r" "opn/cls: %zu/%zu  r/w: %zu/%zu (%llu/%llu)"
+    fprintf(stderr, "\r\e[2K" "opn/cls: %zu/%zu  r/w: %zu/%zu (%llu/%llu)"
       " ioctl: %zu  bad: %zu",
       Stats.open, Stats.close, Stats.read, Stats.write, Stats.in,
       Stats.out, Stats.ioctl, Stats.badpack
@@ -613,6 +534,11 @@ static void sighandler_alrm(int s) {
     return;
 }
 
+static void sighandler_pipe(int s) {
+    fprintf(stderr, "\n" "[%d] Received SIGPIPE\n", getpid());
+    return;
+}
+
 //-----------------------------------------------------------------------------
 static int get_options(int *argc, const char ***argv) {
     static const char *_empty_argv[] = {NULL};
@@ -620,6 +546,8 @@ static int get_options(int *argc, const char ***argv) {
     struct poptOption options_table[] = {
         {NULL, 'D', POPT_ARG_STRING, &Opt.ovdev, 0,
          "Path to the RPL device", "file"},
+        {NULL, 'I', POPT_ARG_NONE, &Opt.infod_start, 0,
+         "Start infod subcomponent", NULL},
         {NULL, 'O', POPT_ARG_STRING, &Opt.ofmt, 0,
          "Override OFMT variable", "string"},
         {NULL, 'Q', POPT_ARG_NONE, &Opt.nocreat, 0,
@@ -629,7 +557,7 @@ static int get_options(int *argc, const char ***argv) {
         {NULL, 'c', POPT_ARG_STRING, &tmp, 'c',
          "Read configuration from file (on top of hardcoded conf)", "file"},
         {NULL, 's', POPT_ARG_NONE, &Opt.syslog, 0,
-         "Use syslog rather than stderr for messages", NULL},
+         "Print warnings/errors to syslog", NULL},
         {NULL, 'v', POPT_ARG_NONE, &Opt.verbose, 0,
          "Print statistics while rpld is running", NULL},
         POPT_AUTOHELP
@@ -661,7 +589,7 @@ static int get_options(int *argc, const char ***argv) {
     }
 
     if((args = poptGetArgs(ctx)) != NULL) {
-        argk = SH_count_args(args);
+        argk = count_args(args);
         poptDupArgv(argk, args, argc, argv);
     } else {
         *argc = 0;
@@ -678,6 +606,9 @@ static int read_config(const char *file) {
         {"FBDEVICE", RCONF_STRING, &Opt.fbdev,  NULL},
         {"OFMT",     RCONF_STRING, &Opt.ofmt,   NULL},
         {"USER",     RCONF_CB,     NULL,        pconfig_user},
+
+        {"INFOD_PORT", RCONF_STRING, &Opt.infod_port, NULL},
+        {"START_INFOD", RCONF_IBOOL, &Opt.infod_start, NULL},
         {NULL},
     };
     return HX_rconfig(file, options_table);
@@ -706,6 +637,12 @@ static int pconfig_user(const char *key, unsigned char type, void *ptr,
 }
 
 //-----------------------------------------------------------------------------
+inline static const char *basename(const char *s) {
+    const char *p = strrchr(s, '/');
+    if(p != NULL) { return p + 1; }
+    return s;
+}
+
 static char *getnamefromuid(uid_t uid, char *result, size_t len) {
     struct passwd ent, *ep;
     char additional[1024];
@@ -724,25 +661,6 @@ static uid_t getuidfromname(const char *name) {
     return ep->pw_uid;
 }
 
-static int notify(int lv, unsigned int opts, const char *fmt, ...) {
-    if(Opt.verbose) {
-        int rv;
-        va_list argp;
-        va_start(argp, fmt);
-        if(opts & NF_NEWLINE) { fprintf(stderr, "\n"); }
-        rv = vfprintf(stderr, fmt, argp);
-        va_end(argp);
-        return rv;
-    } else if(Opt.syslog) {
-        va_list argp;
-        va_start(argp, fmt);
-        vsyslog(lv, fmt, argp);
-        va_end(argp);
-        return 1;
-    }
-    return 0;
-}
-
 static char *replace(char *dest, const char *src, size_t count,
  char **catalog) {
     char *destp = dest;
@@ -759,8 +677,7 @@ static char *replace(char *dest, const char *src, size_t count,
         destp += max;
         count -= max;
         if(*src != '%' || count < 2) { break; }
-        ++src;
-        if(*src == '%') { // "%%"
+        if(*++src == '%') { // "%%"
             *destp++ = '%';
             if(--count < 2) { break; }
             ++src;
@@ -791,7 +708,7 @@ static char *replace(char *dest, const char *src, size_t count,
     return dest;
 }
 
-static int value_cmp(void *pa, void *pb) {
+inline static int value_cmp(void *pa, void *pb) {
     return (pa > pb) - (pa < pb);
 }
 
